#![recursion_limit = "256"]

#[cfg(feature = "ssr")]
extern crate diesel_migrations;

#[cfg(feature = "ssr")]
#[tokio::main]
async fn main() {
    use axum::{extract::Path, middleware::from_fn, routing::get, Router};
    use axum_login::tower_sessions::SessionManagerLayer;
    use axum_login::AuthManagerLayerBuilder;
    use leptos::prelude::*;
    use leptos_axum::{generate_route_list, LeptosRoutes};
    use libretunes::app::*;
    use libretunes::util::auth_backend::AuthBackend;
    use libretunes::util::fileserv::{
        file_and_error_handler, get_asset_file, get_static_file, AssetType,
    };
    use libretunes::util::redis::get_redis_pool;
    use libretunes::util::require_auth::require_auth_middleware;
    use log::*;
    use tower_sessions_redis_store::RedisStore;

    flexi_logger::Logger::try_with_env_or_str("debug")
        .unwrap()
        .format(flexi_logger::opt_format)
        .start()
        .unwrap();

    info!("\n{}", include_str!("../ascii_art.txt"));
    info!("Starting Leptos server...");

    use dotenvy::dotenv;
    dotenv().ok();

    debug!("Running database migrations...");

    // Bring the database up to date
    libretunes::util::database::migrate();

    debug!("Connecting to Redis...");
    let redis_pool = get_redis_pool();
    let session_store = RedisStore::new(redis_pool);
    let session_layer = SessionManagerLayer::new(session_store);

    let auth_backend = AuthBackend;
    let auth_layer = AuthManagerLayerBuilder::new(auth_backend, session_layer).build();

    let conf = get_configuration(None).unwrap();
    let leptos_options = conf.leptos_options;
    let addr = leptos_options.site_addr;
    // Generate the list of routes in your Leptos App
    let routes = generate_route_list(App);

    let app = Router::new()
        .leptos_routes(&leptos_options, routes, {
            let leptos_options = leptos_options.clone();
            move || shell(leptos_options.clone())
        })
        .route(
            "/assets/audio/:song",
            get(|Path(song): Path<String>| get_asset_file(song, AssetType::Audio)),
        )
        .route(
            "/assets/images/:image",
            get(|Path(image): Path<String>| get_asset_file(image, AssetType::Image)),
        )
        .route("/assets/*uri", get(|uri| get_static_file(uri, "")))
        .layer(from_fn(require_auth_middleware))
        .layer(auth_layer)
        .fallback(file_and_error_handler)
        .with_state(leptos_options);

    let listener = tokio::net::TcpListener::bind(&addr)
        .await
        .unwrap_or_else(|_| panic!("Could not bind to {}", &addr));

    info!("Listening on http://{}", &addr);

    axum::serve(listener, app.into_make_service())
        .await
        .expect("Server failed");
}

#[cfg(not(feature = "ssr"))]
pub fn main() {
    // no client-side main function
    // unless we want this to work with e.g., Trunk for pure client-side testing
    // see lib.rs for hydration function instead
}
