use crate::components::dashboard_tile::DashboardTile;
use crate::models::backend::{self, Album, Artist};

use chrono::{NaiveDate, NaiveDateTime};
use serde::{Deserialize, Serialize};

/// Holds information about a song
///
/// Intended to be used in the front-end, as it includes artist and album objects, rather than just their ids.
#[derive(Serialize, Deserialize, Clone)]
pub struct Song {
    /// Song id
    pub id: i32,
    /// Song name
    pub title: String,
    /// Song artists
    pub artists: Vec<Artist>,
    /// Song album
    pub album: Option<Album>,
    /// The track number of the song on the album
    pub track: Option<i32>,
    /// The duration of the song in seconds
    pub duration: i32,
    /// The song's release date
    pub release_date: Option<NaiveDate>,
    /// Path to song file, relative to the root of the web server.
    /// For example, `"/assets/audio/Song.mp3"`
    pub song_path: String,
    /// Path to song image, relative to the root of the web server.
    /// For example, `"/assets/images/Song.jpg"`
    pub image_path: String,
    /// Whether the song is liked by the user
    pub like_dislike: Option<(bool, bool)>,
    /// The date the song was added to the database
    pub added_date: NaiveDateTime,
}

impl TryInto<backend::Song> for Song {
    type Error = Box<dyn std::error::Error>;

    /// Convert a `SongData` object into a Song object
    ///
    /// The SongData/Song conversions are also not truly reversible,
    /// due to the way the `image_path` data is handled.
    fn try_into(self) -> Result<backend::Song, Self::Error> {
        Ok(backend::Song {
            id: self.id,
            title: self.title,
            album_id: self.album.map(|album| album.id),
            track: self.track,
            duration: self.duration,
            release_date: self.release_date,
            storage_path: self.song_path,

            // Note that if the source of the image_path was the album, the image_path
            // will be set to the album's image_path instead of None
            image_path: if self.image_path == "/assets/images/placeholder.jpg" {
                None
            } else {
                Some(self.image_path)
            },

            added_date: self.added_date,
        })
    }
}

impl From<Song> for DashboardTile {
    fn from(val: Song) -> Self {
        DashboardTile {
            image_path: val.image_path.into(),
            title: val.title.into(),
            link: format!("/song/{}", val.id).into(),
            description: Some(format!("Song • {}", Artist::display_list(&val.artists)).into()),
        }
    }
}
