use crate::api::search::search;
use crate::components::dashboard_row::*;
use crate::components::error::*;
use crate::components::loading::*;
use crate::components::song_list::*;
use leptos::html::Input;
use leptos::prelude::*;
use leptos_router::hooks::query_signal;

#[component]
pub fn Search() -> impl IntoView {
    // Sync the search query with the URL parameter "query"
    let (query, set_query) = query_signal::<String>("query");

    let search_sig = RwSignal::new(query.get_untracked().unwrap_or(String::new()));
    Effect::new(move || set_query.set(Some(search_sig.get())));
    Effect::new(move || search_sig.set(query.get().unwrap_or(String::new())));

    let search = Resource::new(
        move || search_sig.get(),
        move |search_sig| search(search_sig, 10),
    );

    let input_ref = NodeRef::<Input>::new();
    input_ref.on_load(move |input| {
        // Select all text in the input field, or just focus it if empty
        input.select();
    });

    view! {
        <input node_ref=input_ref
            class="bg-neutral-800 text-neutral-200 border border-neutral-600 rounded-lg p-2 w-full outline-none"
            type="text" placeholder="Search..." bind:value=search_sig autofocus />

        <Suspense
            fallback=|| view! { <Loading /> }
        >
            <ErrorBoundary
                fallback=|errors| {
                    errors.get().into_iter().map(|(_id, error)| {
                        view! {
                            <Error<String>
                                message=error.to_string()
                            />
                        }
                    }).collect::<Vec<_>>()
                }
            >
                {move || {
                    search.get().map(|results| {
                        results.map(|(albums, artists, songs)| {
                            view! {
                                {
                                    (albums.is_empty() && artists.is_empty() && songs.is_empty()).then(|| {
                                        view! {
                                            <h2 class="text-xl text-neutral-500">"No Results"</h2>
                                        }
                                    })
                                }
                                {
                                    (!albums.is_empty()).then(|| {
                                        view! {
                                            <DashboardRow
                                                title="Albums"
                                                tiles=albums.into_iter().map(|(album, _score)| album.into()).collect()
                                            />
                                        }
                                    })
                                }
                                {
                                    (!artists.is_empty()).then(|| {
                                        view! {
                                            <DashboardRow
                                                title="Artists"
                                                tiles=artists.into_iter().map(|(artist, _score)| artist.into()).collect()
                                            />
                                        }
                                    })
                                }
                                {
                                    (!songs.is_empty()).then(|| {
                                        view! {
                                            <h2 class="text-xl font-bold">"Songs"</h2>
                                            <SongList songs=songs.into_iter().map(|(song, _score)| song).collect() />
                                        }
                                    })
                                }
                            }
                        })
                    })
                }}
            </ErrorBoundary>
        </Suspense>
    }
}
