use leptos::either::*;
use leptos::prelude::*;
use leptos_icons::*;
use leptos_router::hooks::use_params_map;
use server_fn::error::NoCustomError;

use crate::api::songs;
use crate::api::songs::*;
use crate::components::error::*;
use crate::components::loading::*;
use crate::components::song_list::*;
use crate::models::frontend;
use crate::util::state::GlobalState;

use std::borrow::Borrow;
use std::rc::Rc;

#[component]
pub fn SongPage() -> impl IntoView {
    let params = use_params_map();

    view! {
        {move || params.with(|params| {
            match params.get("id").map(|id| id.parse::<i32>()) {
                Some(Ok(id)) => {
                    Either::Left(view! { <SongDetails id /> })
                },
                Some(Err(e)) => {
                    Either::Right(view! {
                        <Error<String>
                            title="Invalid Song ID"
                            error=e.to_string()
                        />
                    })
                },
                None => {
                    Either::Right(view! {
                        <Error<String>
                            title="No Song ID"
                            message="You must specify a song ID to view its page."
                        />
                    })
                }
            }
        })}
    }
}

#[component]
fn SongDetails(#[prop(into)] id: Signal<i32>) -> impl IntoView {
    let song_info = Resource::new(move || id.get(), get_song_by_id);

    view! {
        <Transition
            fallback=move || view! { <LoadingPage /> }
        >
            {move || song_info.get().map(|song| {
                match song {
                    Ok(Some(song)) => {
                        EitherOf3::A(view! { <SongOverview song /> })
                    },
                    Ok(None) => {
                        EitherOf3::B(view! {
                            <Error<String>
                                title="Song Not Found"
                                message=format!("Song with ID {} not found", id.get())
                            />
                        })
                    },
                    Err(error) => {
                        EitherOf3::C(view! {
                            <ServerError<NoCustomError>
                                title="Error Fetching Song"
                                error
                            />
                        })
                    }
                }
            })}
        </Transition>
        <SongPlays id />
        <MySongPlays id />
    }
}

#[component]
fn SongOverview(song: frontend::Song) -> impl IntoView {
    let playing = RwSignal::new(false);
    let icon = Signal::derive(move || {
        if playing.get() {
            icondata::BsPauseFill
        } else {
            icondata::BsPlayFill
        }
    });

    Effect::new(move |_| {
        GlobalState::play_status().with(|status| {
            playing
                .set(status.queue.front().map(|song| song.id) == Some(song.id) && status.playing);
        });
    });

    let song_rc = Rc::new(song.clone());

    let toggle_play_song = move |_| {
        GlobalState::play_status().update(|status| {
            if status.queue.front().map(|song| song.id) == Some(song_rc.id) {
                status.playing = !status.playing;
            } else {
                if let Some(last_playing) = status.queue.front() {
                    status.queue.push_front(last_playing.clone());
                }

                status.queue.clear();
                status.queue.push_front(
                    <Rc<frontend::Song> as Borrow<frontend::Song>>::borrow(&song_rc).clone(),
                );
                status.playing = true;
            }
        });
    };

    view! {
        <div class="flex">
            <div class="relative w-35 h-35">
                <img src=song.image_path />
                <Icon icon on:click={toggle_play_song} {..}
                    class="control w-15 h-15 absolute top-1/2 left-1/2 translate-[-50%]" />
            </div>
            <div class="self-center p-2">
                <h1 class="text-4xl">{song.title}</h1>
                <p><SongArtists artists=song.artists /></p>
                <p><SongAlbum album=song.album /></p>
            </div>
        </div>
        <p>{format!("Duration: {}:{:02}", song.duration / 60, song.duration % 60)}</p>
    }
}

#[component]
fn SongPlays(#[prop(into)] id: Signal<i32>) -> impl IntoView {
    let plays = Resource::new(move || id.get(), songs::get_song_plays);

    view! {
        <Transition
            fallback=move || view! { <Loading /> }
        >
            {move || plays.get().map(|plays| {
                match plays {
                    Ok(plays) => {
                        Either::Left(view! {
                            <p>{format!("Plays: {plays}")}</p>
                        })
                    },
                    Err(error) => {
                        Either::Right(view! {
                            <ServerError<NoCustomError>
                                title="Error fetching song plays"
                                error
                            />
                        })
                    }
                }
            })}
        </Transition>
    }
}

#[component]
fn MySongPlays(#[prop(into)] id: Signal<i32>) -> impl IntoView {
    let plays = Resource::new(move || id.get(), songs::get_my_song_plays);

    view! {
        <Transition
            fallback=move || view! { <Loading /> }
        >
            {move || plays.get().map(|plays| {
                match plays {
                    Ok(plays) => {
                        Either::Left(view! {
                            <p>{format!("My Plays: {plays}")}</p>
                        })
                    },
                    Err(error) => {
                        Either::Right(view! {
                            <ServerError<NoCustomError>
                                title="Error fetching my song plays"
                                error
                            />
                        })
                    }
                }
            })}
        </Transition>
    }
}
